// Window class implementation
#include "pch.h"

#include "Window.h"
#include "util.h"

namespace nf::cli {
    Window::Window()
        : m_handle(nullptr)
        , m_wndClassName("NothinFancyWindow")
        , m_windowedStyle(WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX)
    {
        NFLog("Creating window");

        // Register window class
        WNDCLASS wc = {};
        wc.lpszClassName = m_wndClassName;
        wc.hCursor = LoadCursor(nullptr, IDC_ARROW);
        wc.lpfnWndProc = wndProc;
        RegisterClass(&wc);

        m_handle = CreateWindow(m_wndClassName, "NF Window", m_windowedStyle, 0, 0, 0, 0, nullptr, nullptr, nullptr, this);
    }

    void Window::runLoop() {
        show();
        MSG msg = {};
        while (GetMessage(&msg, nullptr, NULL, NULL)) {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }

    void Window::show(bool show) {
        ShowWindow(m_handle, show ? SW_SHOW : SW_HIDE);
    }

    LRESULT CALLBACK Window::wndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
        static Window* wnd = nullptr;

        switch (msg) {
            case WM_CREATE: {
                wnd = reinterpret_cast<Window*>(reinterpret_cast<CREATESTRUCT*>(lParam)->lpCreateParams);

                // Temporary monitor choice
                POINT cursor = {};
                GetCursorPos(&cursor);
                MONITORINFO mi = {};
                mi.cbSize = sizeof(mi);
                GetMonitorInfo(MonitorFromPoint(cursor, MONITOR_DEFAULTTONEAREST), &mi);
                int x = mi.rcMonitor.left + 100, y = mi.rcMonitor.top + 100;

                RECT cli = {};
                cli.right = 1280;
                cli.bottom = 720;
                AdjustWindowRect(&cli, wnd->m_windowedStyle, FALSE);
                unsigned int width = cli.right - cli.left, height = cli.bottom - cli.top;

                SetWindowPos(hWnd, nullptr, x, y, width, height, SWP_NOZORDER);
                return 0;
            }

            case WM_CLOSE:
                DestroyWindow(hWnd);
                return 0;

            case WM_DESTROY:
                PostQuitMessage(0);
                return 0;
        }

        return DefWindowProc(hWnd, msg, wParam, lParam);
    }

}
